@@ -605,12 +605,14 @@ unity_webapps_indicator_model_set_indicator_property_for_interest (UnityWebappsI
   gboolean changed_property;
 
   g_return_if_fail (UNITY_WEBAPPS_IS_INDICATOR_MODEL (model));
-  
+
+  g_variant_ref_sink (value);
+
   indicator = g_hash_table_lookup (model->priv->indicators_by_name, indicator_name);
   
   if (indicator == NULL)
     {
-      return;
+      goto out;
     }
   
   indicators_for_interest = (GList *)g_hash_table_lookup (model->priv->indicators_for_interest,
@@ -620,11 +622,11 @@ unity_webapps_indicator_model_set_indicator_property_for_interest (UnityWebappsI
   
   if (indicator_link == NULL)
     {
-      return;
+      goto out;
     }
   
   changed_property = 
-    unity_webapps_indicator_model_indicator_push_property (model, indicator, property_name, g_variant_ref (value), interest_id);
+    unity_webapps_indicator_model_indicator_push_property (model, indicator, property_name, value, interest_id);
   
   if (changed_property == TRUE)
     {
@@ -634,6 +636,9 @@ unity_webapps_indicator_model_set_indicator_property_for_interest (UnityWebappsI
       g_signal_emit (model, signals[INDICATOR_PROPERTY_CHANGED], 0, indicator_name,
 		     property_name, value, draw_attention);
     }
+
+ out:
+  g_variant_unref (value);
 }
 
 GVariant *
