@@ -50,8 +50,6 @@ struct _UnityWebappsActionManagerPrivate {
   gboolean track_activity;
 };
 
-typedef struct _UnityWebappsActionInfo UnityWebappsActionInfo;
-
 typedef struct _UnityWebappsActionInfo {
   UnityWebappsActionManager *manager;
   DbusmenuMenuitem *item;
@@ -60,7 +58,7 @@ typedef struct _UnityWebappsActionInfo {
   
   guint ref_count;
   
-  UnityWebappsActionInfo *parent;
+  struct _UnityWebappsActionInfo *parent;
 } UnityWebappsActionInfo;
 
 typedef struct _UnityWebappsHasActionData {
