@@ -18,6 +18,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
  */
 
+#include "config.h"
 #include <stdlib.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -321,7 +322,11 @@ main (gint argc, gchar **argv)
 {
   gchar *name, *domain, *icon_url, *mime_types, *desktop_name;
   gtk_init (&argc, &argv);
-  
+
+  bindtextdomain (GETTEXT_PACKAGE, NULL);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  textdomain (GETTEXT_PACKAGE);
+
   desktop_name = NULL;
   if (!parse_args (argc, argv, &name, &domain, &icon_url, &mime_types, &desktop_name))
     return 1;
@@ -334,11 +339,18 @@ main (gint argc, gchar **argv)
 
   if (desktop_name == NULL)
     {
-      application_info = unity_webapps_application_info_new (name, domain, icon_url, mime_types);
+      application_info = unity_webapps_application_info_new (name,
+                                                             domain,
+                                                             icon_url,
+                                                             mime_types);
     }
   else
     {
-      application_info = unity_webapps_preinstalled_application_info_new (name, domain, icon_url, mime_types, desktop_name);
+      application_info = unity_webapps_preinstalled_application_info_new (name,
+                                                                          domain,
+                                                                          icon_url,
+                                                                          mime_types,
+                                                                          desktop_name);
     }
   
   wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
@@ -437,17 +449,15 @@ on_handle_list_interests (UnityWebappsGenContext *context,
 			  GDBusMethodInvocation *invocation,
 			  gpointer user_data)
 {
-  GVariant *maybe_interests_list, *interests_list;
+  GVariant *interests_list;
   
   UNITY_WEBAPPS_NOTE (CONTEXT, "Handling ListInterests call");
   
-  maybe_interests_list = unity_webapps_interest_manager_list_interests (interest_manager);
+  interests_list = unity_webapps_interest_manager_list_interests (interest_manager);
   
-  if ((maybe_interests_list == NULL) || 
-      ((interests_list = g_variant_get_maybe(maybe_interests_list)) == NULL))
+  if (!interests_list)
     {
       GVariant *array;
-      
       array = g_variant_new ("ai", NULL, 0);
       g_dbus_method_invocation_return_value (invocation, g_variant_new_tuple (&array, 1));
     }
@@ -1155,8 +1165,31 @@ unity_webapps_context_daemon_new ()
 		    G_CALLBACK (on_manager_window_changed),
 		    daemon);
 
-  interest_tracker = unity_webapps_interest_tracker_new (unity_webapps_window_tracker_get_default (), interest_manager);
-  
+  {
+    UnityWebappsWindowTracker*
+      window_tracker;
+
+    // TODO(aau): remove this and make a cleaner abstraction
+    // Keep the #ifdef for now, just to make sure that it does not mess up with
+    // "normal" code path.
+#if defined(ENABLE_TESTS)
+    if (!g_strcmp0(g_getenv("UNITY_WEBAPPS_USE_DBUS_CONTROLLABLE_WINDOW_TRACKING"), "yes"))
+      {
+        UNITY_WEBAPPS_NOTE (CONTEXT, "Using Dbus Controllable window tracker instance");
+
+        window_tracker = unity_webapps_window_tracker_get_dbus_controllable(session_bus);
+      }
+    else
+#endif
+      {
+        UNITY_WEBAPPS_NOTE (CONTEXT, "Using default window tracker instance");
+
+        window_tracker = unity_webapps_window_tracker_get_default();
+      }
+
+    interest_tracker = unity_webapps_interest_tracker_new (window_tracker, interest_manager);
+  }
+
   g_signal_connect (interest_tracker, "notify::most-recent-interest",
 			   G_CALLBACK (unity_webapps_context_daemon_most_recent_interest_changed), daemon);
 		    
@@ -1169,7 +1202,7 @@ unity_webapps_context_daemon_new ()
 
   daemon->notification_context = unity_webapps_notification_context_new (session_bus, name, icon_path);
   daemon->indicator_context = unity_webapps_indicator_context_new (session_bus, unity_webapps_application_info_get_desktop_file_path (application_info), canonical_name, interest_tracker);
-  daemon->music_player_context = unity_webapps_music_player_context_new (session_bus, daemon->desktop_file_name, canonical_name, name);
+  daemon->music_player_context = unity_webapps_music_player_context_new (session_bus, interest_manager, interest_tracker, daemon->desktop_file_name, canonical_name, name);
   daemon->launcher_context = unity_webapps_launcher_context_new (session_bus, daemon->desktop_file_name, interest_tracker, application_info);
   
   g_free (canonical_name);
