@@ -112,6 +112,10 @@ _context_action_invoked (UnityWebappsGenContext *context_proxy,
   unity_webapps_context_action_data *data;
   
   context = (UnityWebappsContext *)user_data;
+  g_return_if_fail(context != NULL);
+
+  if (!G_IS_OBJECT(context))
+      return;
   
   data = (unity_webapps_context_action_data *)g_hash_table_lookup (context->priv->action_callbacks_by_name, label);
   
@@ -146,13 +150,12 @@ unity_webapps_context_daemon_initable_init (GInitable *initable,
 
   if (context->priv->context_name == NULL)
     {
-      
       unity_webapps_gen_service_call_get_context_sync (UNITY_WEBAPPS_GEN_SERVICE (unity_webapps_service_get_proxy (context->priv->service)),
 						       context->priv->name,
 						       context->priv->domain,
 						       context->priv->icon_url ? context->priv->icon_url : "none",
-						       context->priv->mime_types,
-						       &context_name, 
+						       context->priv->mime_types ? context->priv->mime_types : "none",
+						       &context_name,
 						       &version,
 						       cancellable,
 						       error);
@@ -306,7 +309,7 @@ unity_webapps_context_finalize (GObject *object)
     {
       unity_webapps_gen_context_call_lost_interest_sync (context->priv->context_proxy,
 							 context->priv->interest_id,
-               context->priv->user_abandoned,
+                                                         context->priv->user_abandoned,
 							 NULL,
 							 &error);
       
@@ -341,6 +344,7 @@ unity_webapps_context_finalize (GObject *object)
   g_free (context->priv->name);
   g_free (context->priv->domain);
   g_free (context->priv->icon_url);
+  g_free (context->priv->mime_types);
   
   g_free (context->priv->context_name);
   
@@ -516,6 +520,7 @@ unity_webapps_context_init(UnityWebappsContext *context)
   context->priv->name = NULL;
   context->priv->domain = NULL;
   context->priv->icon_url = NULL;
+  context->priv->mime_types = NULL;
   
   context->priv->context_name = NULL;
   context->priv->took_interest = FALSE;
@@ -554,14 +559,21 @@ static gboolean
 complete_in_idle_cb (gpointer user_data)
 {
   unity_webapps_context_new_data *data;
-  
-  data = (unity_webapps_context_new_data *)user_data;
-  
-  data->callback (data->webapps_context, data->user_data);
 
+  data = (unity_webapps_context_new_data *)user_data;
+
+  if (!data || !data->webapps_context)
+    return FALSE;
+
+  // first unref what should be unrefed
   if (data->webapps_object_needs_unref == TRUE)
     g_object_unref (G_OBJECT (data->webapps_context));
 
+  // ... then see what we have left (object might already have been
+  // unrefed by an abandoned for example.
+  if (data->callback && G_IS_OBJECT(data->webapps_context))
+    data->callback (data->webapps_context, data->user_data);
+
   return FALSE;
 }
 
@@ -578,8 +590,8 @@ unity_webapps_context_ready (GObject *source_object,
   
   error = NULL;
   
-  context = UNITY_WEBAPPS_CONTEXT (g_async_initable_new_finish (initable, res, &error)); // TODO: Error
-  
+  context = UNITY_WEBAPPS_CONTEXT (g_async_initable_new_finish (initable, res, &error)); // TODO: Error 
+ 
   if (error != NULL)
     {
       g_critical ("Error in unity_webapps_context_ready: %s", error->message);
@@ -926,9 +938,21 @@ _close_callback (UnityWebappsGenContext *gen_context,
                  gint interest_id,
                  gpointer user_data)
 {
-  unity_webapps_context_close_callback_data *data = (unity_webapps_context_close_callback_data *)user_data;
-  UnityWebappsContext *context = data->context;
-  UnityWebappsContextCloseCallback callback = data->callback;
+  unity_webapps_context_close_callback_data *data;
+  UnityWebappsContext *context;
+  UnityWebappsContextCloseCallback callback;
+
+  data = (unity_webapps_context_close_callback_data *)user_data;
+  g_return_if_fail (data != NULL);
+
+  callback = data->callback;
+  g_return_if_fail (callback != NULL);
+
+  context = data->context;
+  g_return_if_fail (context != NULL);
+
+  if (!G_IS_OBJECT(context))
+      return;
 
   if (context->priv->took_interest == FALSE)
     return;
@@ -946,15 +970,26 @@ _raise_callback (UnityWebappsGenContext *gen_context,
                  const gchar * const *files,
                  gpointer user_data)
 {
-  unity_webapps_context_raise_callback_data *data = (unity_webapps_context_raise_callback_data *)user_data;
-  UnityWebappsContext *context = data->context;
-  UnityWebappsContextRaiseCallback callback = data->callback;
+  unity_webapps_context_raise_callback_data *data;
+  UnityWebappsContext *context;
+  UnityWebappsContextRaiseCallback callback;
   const gchar * const *it;
 
+  data = (unity_webapps_context_raise_callback_data *)user_data;
+  g_return_if_fail (data != NULL);
+
+  callback = data->callback;
+  g_return_if_fail (callback != NULL);
+
+  context = data->context;
+  g_return_if_fail (context != NULL);
+
+  if (!G_IS_OBJECT(context))
+      return;
+
   if (context->priv->took_interest == FALSE)
     return;
 
-
   if ((context->priv->interest_id == interest_id) ||
       (interest_id == UNITY_WEBAPPS_INVALID_INTEREST_ID))
     {
@@ -1111,9 +1146,17 @@ on_notify_view_is_active (UnityWebappsGenContext *context,
 			  gpointer user_data)
 {
   unity_webapps_context_notify_data *data;
-  
+
   data = (unity_webapps_context_notify_data *)user_data;
-  
+  g_return_if_fail (data != NULL);
+  g_return_if_fail (data->context != NULL);
+
+  if (!G_IS_OBJECT(data->context))
+      return;
+
+  if (!data->view_callback)
+    return;
+
   data->view_callback(data->context, interest_id, view_is_active, data->user_data);
 }
 
@@ -1221,9 +1264,17 @@ on_notify_view_location (UnityWebappsGenContext *context,
 			  gpointer user_data)
 {
   unity_webapps_context_notify_data *data;
-  
+
   data = (unity_webapps_context_notify_data *)user_data;
-  
+  g_return_if_fail (data != NULL);
+  g_return_if_fail (data->context != NULL);
+
+  if (!G_IS_OBJECT(data->context))
+      return;
+
+  if (!data->location_callback)
+    return;
+
   data->location_callback(data->context, interest_id, location, data->user_data);
 }
 
@@ -1324,9 +1375,17 @@ on_notify_interests_changed (UnityWebappsGenService *gen_service,
 			    gpointer user_data)
 {
   unity_webapps_context_notify_data *data;
-  
+  UnityWebappsContext *context;
+
   data = (unity_webapps_context_notify_data *)user_data;
-  
+  g_return_if_fail (data != NULL);
+
+  context = data->context;
+  g_return_if_fail (context != NULL);
+
+  if (!G_IS_OBJECT (context))
+    return;
+
   data->callback(data->context, interest_id, data->user_data);
 }
 
@@ -1485,10 +1544,16 @@ on_notify_view_window (UnityWebappsGenContext *context,
 			  gpointer user_data)
 {
   unity_webapps_context_notify_data *data;
-  
+
   data = (unity_webapps_context_notify_data *)user_data;
-  
-  data->window_callback(data->context, interest_id, window, data->user_data);
+  g_return_if_fail (data != NULL);
+  g_return_if_fail (data->context != NULL);
+
+  if (!G_IS_OBJECT (data->context))
+    return;
+
+  if (data->window_callback != NULL)
+    data->window_callback(data->context, interest_id, window, data->user_data);
 }
 
 void 
@@ -1542,10 +1607,19 @@ request_preview_complete_callback (GObject *source_object,
   UnityWebappsGenContext *context_proxy;
   gchar *preview_data;
   unity_webapps_context_notify_data *data;
+  UnityWebappsContext *context;
   GError *error;
 
   context_proxy = UNITY_WEBAPPS_GEN_CONTEXT (source_object);
+
   data = (unity_webapps_context_notify_data *)user_data;
+  g_return_if_fail (data != NULL);
+
+  context = data->context;
+  g_return_if_fail (context != NULL);
+
+  if (!G_IS_OBJECT(context))
+      return;
   
   error = NULL;
   
@@ -1562,7 +1636,8 @@ request_preview_complete_callback (GObject *source_object,
       return;
     }
   
-  data->preview_callback (data->context, data->interest_id, preview_data, data->user_data);
+  if (data->preview_callback)
+    data->preview_callback (data->context, data->interest_id, preview_data, data->user_data);
   
   g_free (data);
   g_free (preview_data);
