@@ -13,7 +13,8 @@ static void
 setup_fixture_simple (AvailableApplicationTestFixture *fixture, gconstpointer user_data)
 {
   fixture->app = 
-    (UnityWebappsLocalAvailableApplication *) unity_webapps_local_available_application_new (unity_webapps_application_manifest_new_from_file ("test-manifest-1.json"));
+    (UnityWebappsLocalAvailableApplication *) unity_webapps_local_available_application_new (
+        unity_webapps_application_manifest_new_from_file (TESTDATADIR "test-manifest-1.json"));
 }
 
 
