@@ -23,6 +23,13 @@
 
 #include <stdlib.h>
 
+// in sync with what is in src/context-daemon/interest-tracking/unity-webapps-window-tracker-dbus-controllable.c
+#define WINDOW_TRACKER_DBUS_CONTROLLABLE_DBUS_PATH \
+  "com.canonical.Unity.Webapps.Tests.WindowTracker.DbusControllable"
+#define WINDOW_TRACKER_DBUS_CONTROLLABLE_DBUS_SIGNAL \
+  "UpdateLatestWindowId"
+
+
 void
 uwa_emit_test_finished ()
 {
@@ -42,3 +49,20 @@ uwa_emit_test_finished ()
   
   exit(0);
 }
+
+void uwa_emit_window_changed (guint64 window_id)
+{
+  GDBusConnection *connection;
+  connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL); // TODO: Error
+
+  g_dbus_connection_emit_signal (connection,
+				 NULL,
+				 "/",
+				 WINDOW_TRACKER_DBUS_CONTROLLABLE_DBUS_PATH,
+				 WINDOW_TRACKER_DBUS_CONTROLLABLE_DBUS_SIGNAL,
+				 g_variant_new ("(t)", window_id, NULL),
+				 NULL);
+
+  g_dbus_connection_flush_sync (connection, NULL, NULL);
+}
+
