@@ -36,7 +36,13 @@ main (gint argc, gchar **argv)
   
   service = unity_webapps_service_new ();
   
-  unity_webapps_context_new (service, "Test", "test.ts", ICON_URL, NULL, context_ready, NULL);
+  unity_webapps_context_new (service,
+                             "TestIndicatorRaiseCallback",
+                             "test.ts",
+                             ICON_URL,
+                             NULL,
+                             context_ready,
+                             NULL);
   
   mainloop = g_main_loop_new (NULL, FALSE);
   
