@@ -1,7 +1,12 @@
 #!/bin/bash
 set -e 
+
+if [ -z $UNITY_WEBAPPS_DAEMON_EXEC_DIR ]; then
+    export UNITY_WEBAPPS_DAEMON_EXEC_DIR="`pwd`/../../src/context-daemon/"
+fi
+
 if [ -d test-output-files ]; then
-    rm -r test-output-files
+    rm -fR test-output-files
 fi
 
 mkdir test-output-files
@@ -12,4 +17,4 @@ export UNITY_WEBAPPS_CONTEXT_ICON_THEME_DIR="test-output-files"
 export UNITY_WEBAPPS_CONTEXT_ICON_APPLICATION_DIR="test-output-files"
 export UNITY_WEBAPPS_CONTEXT_ICON_RESOURCE_DIR="test-output-files"
 test-runner/test-runner -n $1 $2 $3
-rm -r test-output-files
+rm -fR test-output-files
